package info.textgrid.middleware;

import java.io.IOException;
import java.text.ParseException;
import java.util.Hashtable;
import java.util.Map;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.search.SearchScrollRequest;
import org.elasticsearch.action.search.SearchType;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.common.Strings;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import info.textgrid.middleware.oaipmh.ListIdentifiersType;

/**
 * <p>
 * Class to build the Element for a ListIdentifiers request.
 * </p>
 * 
 * @author Maximilian Brodhun, SUB Göttingen
 * @author Stefan E. Funk, SUB Göttingen
 * @version 2021-10-25
 * @since 2014-02-20
 */

public class IdentifierListDelivererDC extends IdentifierListDelivererAbstract {

  protected static Map<String, Integer> cursorCollector = new Hashtable<String, Integer>();

  /**
   * <p>
   * In OAIPMH a ListIdentifiers request is answered by responding the datestamp and the identifier
   * of an object Both will be responded in a header element. This function build the header
   * element.
   * </p>
   * 
   * @param datestamp taken from the elasticSearch index (created element)
   * @param identifier taken from the elasticSearch index (textgridUri)
   * @return the ListIdentifierElement with the responded header element
   */
  public IdentifierListDelivererDC(boolean textgrid, boolean dariah) {
    super(textgrid, dariah);
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.IdentifierListDelivererInterface#processIdentifierList(java.lang.
   * String, java.lang.String, java.lang.String, java.lang.String)
   */
  public ListIdentifiersType processIdentifierList(String from, String to, String set,
      String resumptionToken) throws IOException, ParseException {

    /*
     * To get the required values for the ListIdentifiers request this function will ask
     * ElasticSearc for a specific textgridUri the values "created" and "textgridUri".
     */

    /*
     * Since the ListIdentifiers request enables the possibility the limit the request in a specific
     * time interval, it is necessary to perform a range query on the ElasticSearch index.
     */

    // FIXME combine it with with getIdentifierListWithSet and check for empty set

    ListIdentifiersType lit = new ListIdentifiersType();
    QueryBuilder query = setOrNot(set, from, to);
    SearchResponse listListIdentiferValues;

    String[] includes = this.identifierListFields;
    String[] excludes = Strings.EMPTY_ARRAY;
    // TODO: necessary?
    // FetchSourceContext fetchSourceContext = new FetchSourceContext(true, includes, excludes);

    SearchRequest searchRequest =
        new SearchRequest(OAI_ESClient.getEsIndex()).searchType(SearchType.QUERY_THEN_FETCH);

    SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
    searchSourceBuilder.query(query);
    searchSourceBuilder.size(this.searchResponseSize);
    searchSourceBuilder.fetchSource(includes, excludes);

    if (resumptionToken != null) {
      SearchScrollRequest scrollRequest = new SearchScrollRequest(resumptionToken);
      scrollRequest.scroll(TimeValue.timeValueSeconds(lifeTimeResToken));
      listListIdentiferValues =
          OAI_ESClient.getEsClient().scroll(scrollRequest, RequestOptions.DEFAULT);
    } else {
      searchRequest.source(searchSourceBuilder);
      searchRequest.scroll(TimeValue.timeValueMinutes(lifeTimeResToken));
      listListIdentiferValues =
          OAI_ESClient.getEsClient().search(searchRequest, RequestOptions.DEFAULT);
    }

    listListIdentiferValues = hitHandling(listListIdentiferValues, lit, set,
        listListIdentiferValues.getScrollId(), cursorCollector);

    return lit;
  }

}
