package info.textgrid.middleware;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.text.ParseException;
import java.util.Hashtable;
import java.util.Map;
import javax.xml.datatype.DatatypeConfigurationException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.elasticsearch.action.get.GetRequest;
import org.elasticsearch.action.get.GetResponse;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.search.SearchScrollRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.Strings;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.FetchSourceContext;
import org.json.JSONObject;
import info.textgrid.middleware.oaipmh.HeaderType;
import info.textgrid.middleware.oaipmh.ListRecordsType;
import info.textgrid.middleware.oaipmh.MetadataType;
import info.textgrid.middleware.oaipmh.RecordType;
import info.textgrid.middleware.oaipmh.ResumptionTokenType;

/**
 *
 */
public class RecordListDelivererDC extends RecordListDelivererAbstract {

  // **
  // FINALS
  // **

  private static final int LIFETIME_RES_TOKEN = 600;

  // **
  // STATICS
  // **

  private static Log log = LogFactory.getLog(RecordListDelivererDC.class);
  protected static Map<String, Integer> cursorCollector = new Hashtable<String, Integer>();

  // **
  // DC-Field Lists
  // **

  private String[] contributorList;
  private String[] coverageList;
  private String[] creatorList;
  private String[] dateList;
  private String[] descriptionList;
  private String[] formatList;
  private String[] identifierList;
  private String[] languageList;
  private String[] publisherList;
  private String[] relationList;
  private String[] relationsForWorkList;
  private String[] rightsList;
  private String[] sourceList;
  private String[] subjectList;
  private String[] titleList;
  private String[] typeList;

  /**
   * @param textgrid
   * @param dariah
   */
  public RecordListDelivererDC(boolean textgrid, boolean dariah) {
    super(textgrid, dariah);
  }

  /**
   * <p>
   * The furtherDCElementes fetches the specified fields from the work object. This function is
   * called from the function fetchFields.
   * </p>
   * 
   * @param id
   * @param client
   * @return
   */
  public GetResponse furtherDCElements(String id, RestHighLevelClient client) {

    if (id != null) {
      try {
        id = URLDecoder.decode(id, "UTF-8");
      } catch (UnsupportedEncodingException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
      String[] includes = this.workFields;
      String[] excludes = Strings.EMPTY_ARRAY;
      FetchSourceContext fetchSourceContext = new FetchSourceContext(true, includes, excludes);
      GetRequest getRequest =
          new GetRequest(OAI_ESClient.getEsIndex(), OAI_ESClient.getEsType(), id)
              .fetchSourceContext(fetchSourceContext);

      GetResponse responseWorkValues = null;
      try {
        responseWorkValues = OAI_ESClient.getEsClient().get(getRequest, RequestOptions.DEFAULT);
      } catch (IOException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }

      return responseWorkValues;
    }

    return null;
  }

  /**
   * <p>
   * The fetchFields function filters all edition and fetch the specified fields.
   * </p>
   * 
   * @param query
   * @param recordList
   * @param resumptionToken
   * @param set
   * @throws UnsupportedEncodingException
   */
  public void fetchFields(QueryBuilder query, ListRecordsType recordList, String resumptionToken,
      String set) throws UnsupportedEncodingException {

    String scrollID = "";
    SearchResponse scrollResp;
    QueryBuilder recordFilter;
    DublinCoreBuilder dublinCoreBuilder = new DublinCoreBuilder();

    log.debug("formatField: " + (this.formatField == null ? "NULL" : this.formatField));
    log.debug("formatToFilter: " + (this.formatToFilter == null ? "NULL" : this.formatToFilter));

    if (this.dariah == true) {
      // All objects!
      recordFilter = query;
    } else {
      // All editions only!
      recordFilter = QueryBuilders.boolQuery().must(query)
          .must(QueryBuilders.matchPhraseQuery(this.formatField, this.formatToFilter));
    }

    SearchRequest searchRequest = new SearchRequest(OAI_ESClient.getEsIndex());
    SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();

    scrollResp = null;

    searchSourceBuilder.query(recordFilter);
    searchSourceBuilder.size(100);
    searchRequest.source(searchSourceBuilder);

    if (resumptionToken != null) {
      SearchScrollRequest scrollRequest = new SearchScrollRequest(resumptionToken);
      scrollRequest.scroll(TimeValue.timeValueSeconds(LIFETIME_RES_TOKEN));

      try {
        scrollResp = OAI_ESClient.getEsClient().scroll(scrollRequest, RequestOptions.DEFAULT);
      } catch (IOException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    } else {
      searchRequest.source(searchSourceBuilder);
      searchRequest.scroll(TimeValue.timeValueMinutes(LIFETIME_RES_TOKEN));
      try {
        scrollResp = OAI_ESClient.getEsClient().search(searchRequest, RequestOptions.DEFAULT);
      } catch (IOException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    }

    log.info("Queried fields: " + this.fields.toString());
    log.info("Lifetime for resumption token is set to: " + LIFETIME_RES_TOKEN);

    scrollID = scrollResp.getScrollId();

    long completeListSize = scrollResp.getHits().totalHits;
    setResultSize(completeListSize);

    if (completeListSize > 0) {
      setFoundItems(true);

      int i = 0;
      for (SearchHit hit : scrollResp.getHits().getHits()) {
        i++;

        if (hit != null && hit.getFields() != null) {
          JSONObject json = new JSONObject(hit.getSourceAsMap());
          this.modifiedValue = OAIPMHUtilities.fieldLoader(json, this.modifiedField);

          // **
          // TEXTGRID SEARCH
          // **

          if (this.textgrid) {
            String workUri = "";
            String[] relationFields = new String[] {this.relationToFurtherMetadataObject};

            if (hit.getSourceAsMap().get(this.formatField).toString().equals(this.formatToFilter)) {
              String setSpec = this.specFieldPrefix
                  + DublinCoreFieldLoader.fillList(hit, TGConstants.RELATIONS_LIST).get(0);
              if (DublinCoreFieldLoader.fillList(hit, relationFields).get(0) != null) {
                workUri = DublinCoreFieldLoader.fillList(hit, relationFields).get(0)
                    .substring(this.repositoryObjectURIPrefix.length());

                if (!workUri.endsWith(".0")) {
                  workUri = workUri.concat(".0");
                }
              }
              try {
                dublinCoreBuilder = putContentIntoDCFieldListsTG(hit,
                    furtherDCElements(workUri, OAI_ESClient.getEsClient()));

                log.debug(
                    "ID field: " + (this.identifierField == null ? "NULL" : this.identifierField));

                String identifier = hit.getSourceAsMap().get(this.identifierField).toString();

                log.debug("identifier (tgrep): " + identifier);
                log.debug("setSpec: " + setSpec);

                HeaderType header = OAIPMHUtilities.computeTheF______Header(
                    OAIPMHUtilities.convertDateFormat(this.modifiedValue).toString(), identifier,
                    setSpec);
                buildRecord(recordList, header, dublinCoreBuilder);

              } catch (ParseException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
              } catch (DatatypeConfigurationException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
              }
            }
          }

          // **
          // DARIAH SEARCH
          // **

          else if (this.dariah) {

            // Get identifier and setSpec field, add setSpec prefix if not already set.
            String identifier = OAIPMHUtilities.fieldLoader(json, this.identifierField);
            String setSpec = OAIPMHUtilities.getSetSpec(set, this.specFieldPrefix, identifier);

            log.debug("identifier (dhrep): " + identifier);
            log.debug("setSpec: " + set);

            try {
              dublinCoreBuilder = putContentIntoDCFieldListsDH(hit);

              HeaderType header = OAIPMHUtilities.computeTheF______Header(
                  OAIPMHUtilities.convertDateFormat(this.modifiedValue).toString(), identifier,
                  setSpec);
              buildRecord(recordList, header, dublinCoreBuilder);

            } catch (ParseException e) {
              // TODO Auto-generated catch block
              e.printStackTrace();
            } catch (DatatypeConfigurationException e) {
              // TODO Auto-generated catch block
              e.printStackTrace();
            }
          }
        }
      }

      // Check the need for a resumption token!
      ResumptionTokenType resTokenForResponse = OAIPMHUtilities.getResumptionToken(completeListSize,
          resumptionToken, cursorCollector, scrollID, this.searchResponseSize, i);
      recordList.setResumptionToken(resTokenForResponse);

      log.debug("RESTOK: " + recordList.getResumptionToken());

    } else {
      setFoundItems(false);
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.RecordListDelivererInterface#getRecords(java.lang.String,
   * java.lang.String, java.lang.String, java.lang.String)
   */
  public ListRecordsType getRecords(String from, String to, String set, String resumptionToken)
      throws ParseException, IOException {

    log.info("Started List Records with booleans for DARIAH: " + this.dariah + " and TextGrid: "
        + this.textgrid);

    ListRecordsType recordList = new ListRecordsType();
    QueryBuilder query;
    QueryBuilder rangeQuery = QueryBuilders.rangeQuery(this.rangeField).from(from).to(to);

    if (set != null) {
      String queryField = "";
      String valueField = "";

      // **
      // DARIAH
      // **

      if (this.dariah == true) {
        // FIXME Put query field into config!
        queryField = "administrativeMetadata.dcterms:relation";
        valueField = set;
      }

      // **
      // TEXTGRID
      // **

      if (this.textgrid == true) {
        String[] setParts = set.split(":");
        if (setParts[0].equals("project")) {
          // FIXME Put query field into config!
          queryField = "project.id";
          valueField = setParts[1];
        }
      }

      QueryBuilder matchQuery = QueryBuilders.matchPhraseQuery(queryField, valueField);
      QueryBuilder boolQuery = QueryBuilders.boolQuery().must(rangeQuery).must(matchQuery);

      query = boolQuery;
    } else {
      query = rangeQuery;
    }

    try {

      log.debug("getRecords(): " + (query != null ? "queryName=" + query.getName() : "query=NULL")
          + " | " + (recordList != null ? recordList : "recordList=NULL") + " | "
          + (resumptionToken != null ? resumptionToken : "resumptionToken=NULL") + " | "
          + (set != null ? set : "set=NULL"));

      fetchFields(query, recordList, resumptionToken, set);

      // FOR DHREP ONLY: Add record with set metadata in it (to also find collection metadata in
      // Repository Search!)
      // FIXME Put query field into config!
      if (this.dariah && set != null && !set.isEmpty()) {
        QueryBuilder setQuery = QueryBuilders
            .matchQuery("administrativeMetadata.dcterms:identifier", this.specFieldPrefix + set);

        fetchFields(QueryBuilders.boolQuery().must(setQuery), recordList, resumptionToken, set);
      }

    } catch (UnsupportedEncodingException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

    return recordList;
  }

  // **
  // PRIVATE METHIODS
  // **

  /**
   * @param recordList
   * @param header
   * @param dublinCoreBuilder
   */
  private static void buildRecord(ListRecordsType recordList, HeaderType header,
      DublinCoreBuilder dublinCoreBuilder) {

    // Get metadata.
    MetadataType metadata = new MetadataType();
    metadata = dublinCoreBuilder.getDC();

    // Build record, set header and metadata.
    RecordType record = new RecordType();
    record.setHeader(header);
    record.setMetadata(metadata);

    // Add record to the given list.
    recordList.getRecord().add(record);
  }

  /**
   * @param hit
   * @param responseWorkValues
   * @return
   * @throws DatatypeConfigurationException
   * @throws ParseException
   */
  private DublinCoreBuilder putContentIntoDCFieldListsTG(SearchHit hit,
      GetResponse responseWorkValues) throws ParseException, DatatypeConfigurationException {

    DublinCoreBuilder result = new DublinCoreBuilder();

    result.setContributor(
        DublinCoreFieldLoader.fillListFromTGWorkValues(responseWorkValues, this.contributorList));
    result.setCoverage(
        DublinCoreFieldLoader.fillListFromTGWorkValues(responseWorkValues, this.coverageList));
    result.setCreator(
        DublinCoreFieldLoader.fillListFromTGWorkValues(responseWorkValues, this.creatorList));
    result
        .setDate(DublinCoreFieldLoader.fillListFromTGWorkValues(responseWorkValues, this.dateList));
    result.setDescription(
        DublinCoreFieldLoader.fillListFromTGWorkValues(responseWorkValues, this.descriptionList));
    result.setFormat(DublinCoreFieldLoader.fillList(hit, this.formatList));
    result.setIdentifier(
        OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), this.identifierList));
    result.setLanguage(DublinCoreFieldLoader.fillList(hit, this.languageList));
    result.setPublisher(DublinCoreFieldLoader.fillList(hit, this.publisherList));
    result.setRelation(DublinCoreFieldLoader.fillList(hit, this.relationList));
    result.setRelation(DublinCoreFieldLoader.fillListFromTGWorkValues(responseWorkValues,
        this.relationsForWorkList));
    result.setRights(DublinCoreFieldLoader.fillList(hit, this.rightsList));
    result.setSource(DublinCoreFieldLoader.fillList(hit, this.sourceList));
    result.setSubject(DublinCoreFieldLoader.fillList(hit, this.subjectList));
    result.setTitle(DublinCoreFieldLoader.fillList(hit, this.titleList));
    result
        .setType(DublinCoreFieldLoader.fillListFromTGWorkValues(responseWorkValues, this.typeList));

    return result;
  }

  /**
   * @param hit
   * @return
   * @throws ParseException
   * @throws DatatypeConfigurationException
   */
  private DublinCoreBuilder putContentIntoDCFieldListsDH(SearchHit hit)
      throws ParseException, DatatypeConfigurationException {

    DublinCoreBuilder result = new DublinCoreBuilder();

    result.setContributor(
        OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), this.contributorList));
    result.setCoverage(
        OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), this.coverageList));
    result.setCreator(
        OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), this.creatorList));
    result
        .setDate(OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), this.dateList));
    result.setDescription(
        OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), this.descriptionList));
    result.setFormat(
        OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), this.formatList));
    result.setIdentifier(
        OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), this.identifierList));
    result.setLanguage(
        OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), this.languageList));
    result.setPublisher(
        OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), this.publisherList));
    result.setRelation(
        OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), this.relationList));
    result.setRights(
        OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), this.rightsList));
    result.setSource(
        OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), this.sourceList));
    result.setSubject(
        OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), this.subjectList));
    result.setTitle(
        OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), this.titleList));
    result
        .setType(OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), this.typeList));

    return result;
  }

  // **
  // GETTERS AND SETTERS for DublinCore Lists
  // **

  /**
   * @param contributorList
   */
  public void setContributors(String[] contributorList) {
    this.contributorList = contributorList;
  }

  /**
   * @param coveragesList
   */
  public void setCoverages(String[] coveragesList) {
    this.coverageList = coveragesList;
  }

  /**
   * @param creatorsList
   */
  public void setCreators(String[] creatorsList) {
    this.creatorList = creatorsList;
  }

  /**
   * @param datesList
   */
  public void setDates(String[] datesList) {
    this.dateList = datesList;
  }

  /**
   * @param descriptionsList
   */
  public void setDescriptions(String[] descriptionsList) {
    this.descriptionList = descriptionsList;
  }

  /**
   * @param formatsList
   */
  public void setFormats(String[] formatsList) {
    this.formatList = formatsList;
  }

  /**
   * @param identifiersList
   */
  public void setIdentifiers(String[] identifiersList) {
    this.identifierList = identifiersList;
  }

  /**
   * @param languagesList
   */
  public void setLanguages(String[] languagesList) {
    this.languageList = languagesList;
  }

  /**
   * @param publishersList
   */
  public void setPublishers(String[] publishersList) {
    this.publisherList = publishersList;
  }

  /**
   * @param rightsList
   */
  public void setRights(String[] rightsList) {
    this.rightsList = rightsList;
  }

  /**
   * @param sourcesList
   */
  public void setSources(String[] sourcesList) {
    this.sourceList = sourcesList;
  }

  /**
   * @param subjectsList
   */
  public void setSubjects(String[] subjectsList) {
    this.subjectList = subjectsList;
  }

  /**
   * @param titlesList
   */
  public void setTitles(String[] titlesList) {
    this.titleList = titlesList;
  }

  /**
   * @param typesList
   */
  public void setTypes(String[] typesList) {
    this.typeList = typesList;
  }

  /**
   * @param relationsList
   */
  public void setRelations(String[] relationsList) {
    this.relationList = relationsList;
  }

  /**
   * @param relationsForWorkList
   */
  public void setRelationsForWork(String[] relationsForWorkList) {
    this.relationsForWorkList = relationsForWorkList;
  }

}
