package info.textgrid.middleware;

import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import javax.xml.bind.JAXBException;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.XMLGregorianCalendar;
import org.json.JSONObject;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import info.textgrid.middleware.common.TextGridMimetypes;
import info.textgrid.middleware.oaipmh.DeletedRecordType;
import info.textgrid.middleware.oaipmh.GranularityType;

/**
 * @author Maximilian Brodhun, SUB Göttingen
 * @author Stefan E. Funk, SUB Göttingen
 */

public class OaiPmhTestTG {

  // **
  // STATICS
  // **

  public static OAI_ESClient oaiEsClient;
  private static RecordDelivererDC record;
  private static RecordDelivererIDIOM recordIDIOM;
  // private static OpenAireRecord openAireRecord;
  private static RecordDelivererDATACITE recordDATACITE;
  // private static OpenAireRecordList openAireRecordList;
  private static RecordListDelivererDATACITE recordListDATACITE;
  // private static OpenAireIdentifierList openAireIdentifierList;
  private static IdentifierListDelivererDATACITE identifierListDATACITE;
  private static RecordListDelivererDC recordList;
  private static RecordListDelivererIDIOM recordListIDIOM;
  private static IdentifierListDelivererDC identifierList =
      new IdentifierListDelivererDC(true, false);
  private static IdentifierListDelivererIDIOM identifierListIDIOM =
      new IdentifierListDelivererIDIOM(true, false);
  private static SetListDeliverer setListTextGrid;

  // **
  // CLASS
  // **

  private DeletedRecordType bla = DeletedRecordType.NO;
  private GranularityType bla2 = GranularityType.YYYY_MM_DD_THH_MM_SS_Z;
  private RepIdentification rep = new RepIdentification("TextGrid-Repository", "www.textgridrep.de",
      this.bla, "2011-06-11T02:32:40Z", this.bla2, "2.0", "textgrid-support@gwdg.de");
  private MetadataFormatListDelivererInterface metadataFormatList =
      new MetadataFormatListDelivererTG();
  private OAIPMHImpl request = new OAIPMHImpl(this.rep,
      OaiPmhTestTG.record,
      OaiPmhTestTG.recordIDIOM,
      OaiPmhTestTG.recordDATACITE,
      OaiPmhTestTG.recordList,
      OaiPmhTestTG.recordListIDIOM,
      OaiPmhTestTG.recordListDATACITE,
      this.metadataFormatList,
      OaiPmhTestTG.setListTextGrid,
      OaiPmhTestTG.identifierList,
      OaiPmhTestTG.identifierListIDIOM,
      // OaiPmhTestTG.openAireIdentifierList);
      OaiPmhTestTG.identifierListDATACITE);

  OAIPMHUtilities settings = new OAIPMHUtilities();

  /**
   * @throws Exception
   */
  @BeforeClass
  public static void setUp() throws Exception {
    int[] ports = new int[] {9202};
    oaiEsClient = new OAI_ESClient("localhost", ports, 100);
    OAI_ESClient.setEsIndex("textgrid-public");
    OAI_ESClient.setEsType("metadata");

    OAI_ESClient.setPorts(ports);
    record = new RecordDelivererDC(true, false);
    record.setOaiEsClient(oaiEsClient);
    record.setWorkFields(TGConstants.TGWorkFields);

    recordIDIOM = new RecordDelivererIDIOM(true, false);
    recordIDIOM.setOaiEsClient(oaiEsClient);

    /*
     * openAireRecord = new OpenAireRecord(true, false); openAireRecord.setOaiEsClient(oaiEsClient);
     * 
     * openAireRecordList = new OpenAireRecordList(true, false);
     * 
     * openAireIdentifierList = new OpenAireIdentifierList(true, false);
     */

    recordList = new RecordListDelivererDC(true, false);
    recordList.setFields(TGConstants.TGFields);
    recordList.setWorkFields(TGConstants.TGWorkFields);
    recordList.setOaiEsClient(oaiEsClient);
    recordList.setSearchResponseSize(100);

    recordListIDIOM = new RecordListDelivererIDIOM(true, false);
    recordListIDIOM.setOaiEsClient(oaiEsClient);

    identifierList = new IdentifierListDelivererDC(true, false);
    identifierList.setOaiEsClient(oaiEsClient);
    identifierList.setIdentifierListFields(TGConstants.IDENTIFIER_LIST_FIELDS);

    setListTextGrid = new SetListDeliverer(true, false);
  }


  /**
   * 
   */
  @Test
  public void testGetArrayFromESIndex() {
    String jsonAsString =
        "{\"administrativeMetadata\":{\"dcterms:identifier\":\"hdl:21.T11991/0000-001B-4C02-C\",\"dcterms:modified\":\"2021-06-08T11:03:37.448\"},\"descriptiveMetadata\":{\"dc:creator\":\"fu\",\"dc:format\":\"text/vnd.dariah.dhrep.collection+turtle\",\"dc:rights\":\"free\",\"dc:title\":\"First ES6 Tests for DH-rep using Publikator\",\"dc:identifier\":[\"hdl:21.T11991/0000-001B-4C02-C\",\"doi:10.20375/0000-001B-4C02-C\"]}}";

    JSONObject json = new JSONObject(jsonAsString);
    // System.out.println(json);
    DublinCoreBuilder result = new DublinCoreBuilder();
    List<String> identifier = new ArrayList<String>();
    String[] fieldsForIdentifier = {"descriptiveMetadata.dc:identifier"};
    System.out.println(OAIPMHUtilities.fieldLoader(json, OAIPMHConstants.TEST_DARIAH_IDENTIFIER));
    identifier = OAIPMHUtilities.fieldLoader(json, fieldsForIdentifier);
    for (String id : identifier) {
      System.out.println(id);
    }

    result.setIdentifier(identifier);
  }

  /**
   * 
   */
  @Test
  public void testGetArrayFromESIndexForListRecords() {
    String jsonAsString = "{\r\n"
        + "        \"administrativeMetadata\": {\r\n"
        + "            \"dcterms:created\": \"2021-06-16T17:15:48.689\",\r\n"
        + "            \"dcterms:creator\": \"StefanFunk@dariah.eu\",\r\n"
        + "            \"dcterms:extent\": 154064,\r\n"
        + "            \"dcterms:identifier\": \"hdl:21.T11991/0000-001B-4D41-4\",\r\n"
        + "            \"dcterms:modified\": \"2021-06-16T17:15:48.689\"\r\n"
        + "        },\r\n"
        + "        \"descriptiveMetadata\": {\r\n"
        + "            \"dc:creator\": \"fu\",\r\n"
        + "            \"dc:description\": \"just some ictures\",\r\n"
        + "            \"dc:format\": \"image/jpeg\",\r\n"
        + "            \"dc:identifier\": [\r\n"
        + "                \"hdl:21.T11991/0000-001B-4D41-4\",\r\n"
        + "                \"doi:10.20375/0000-001B-4D41-4\"\r\n"
        + "            ],\r\n"
        + "            \"dc:relation\": \"hdl:21.T11991/0000-001B-4D28-1\",\r\n"
        + "            \"dc:rights\": \"free\",\r\n"
        + "            \"dc:title\": \"2017-07-12-Waffel copy 2.jpeg\"\r\n"
        + "        },\r\n"
        + "        \"technicalMetadata\": {\r\n"
        + "            \"externalIdentifier\": \"fmt/43\",\r\n"
        + "            \"externalIdentifierType\": \"puid\",\r\n"
        + "            \"format\": \"JPEG File Interchange Format\",\r\n"
        + "            \"mimetype\": \"image/jpeg\",\r\n"
        + "            \"valid\": \"true\",\r\n"
        + "            \"version\": \"1.01\",\r\n"
        + "            \"well-formed\": \"true\"\r\n"
        + "        }\r\n"
        + "    }";

    JSONObject json = new JSONObject(jsonAsString);
    // System.out.println(json);
    DublinCoreBuilder result = new DublinCoreBuilder();
    List<String> identifier = new ArrayList<String>();
    String[] fieldsForIdentifier = {"descriptiveMetadata.dc:identifier"};
    System.out.println(OAIPMHUtilities.fieldLoader(json, OAIPMHConstants.TEST_DARIAH_IDENTIFIER));
    identifier = OAIPMHUtilities.fieldLoader(json, fieldsForIdentifier);
    for (String id : identifier) {
      System.out.println(id);
    }

    result.setIdentifier(identifier);

    System.out.println(result);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testOpenAireIdentifierList() {

    this.settings.setRangeField(TGConstants.RANGE_FIELD);
    this.settings.setFormatField(TGConstants.FORMAT);
    this.settings.setFormatToFilter(TextGridMimetypes.EDITION);
    this.settings.setSearchResponseSize(100);

    String p =
        this.request.getRequest("ListIdentifiers", "", "oai_datacite", "", "2013", "2014", "");

    System.out.println(p);
  }

  /**
   * @throws DatatypeConfigurationException
   */
  @Test
  @Ignore
  public void testDateNow() throws DatatypeConfigurationException {
    System.out.println("---------------Test Datestamp Parsing with now date -----------------");
    XMLGregorianCalendar nowTest = OAIPMHUtilities.getXMLGregorianCalendarNow();
    System.out.println(nowTest);
    System.out.println("-------------------------------------------");
  }

  /**
   * @throws ParseException
   * @throws DatatypeConfigurationException
   */
  @Test
  @Ignore
  public void testDateGregorianType() throws ParseException, DatatypeConfigurationException {
    String dateformatbefore = "2012-02-06T20:48:39.614+01:00";
    System.out.println("--------- Test Datestamp Parsing with a given date ---------------");
    System.out.println("Original date: " + dateformatbefore);
    XMLGregorianCalendar testDate = OAIPMHUtilities.convertDateFormat(dateformatbefore);
    System.out.println("Date after conversion: " + testDate);
    System.out.println("---------------------------------------------");
  }

  /**
   * @throws ParseException
   * @throws DatatypeConfigurationException
   */
  @Test
  @Ignore
  public void testDateStringType() throws ParseException, DatatypeConfigurationException {
    String dateformatbefore = "2012-02-06T20:48:39.614+01:00";
    System.out.println("------------String Version-------------------");
    System.out.println("Original date: " + dateformatbefore);
    String testDate = OAIPMHUtilities.datestampAsString(dateformatbefore);
    System.out.println("Date after conversion: " + testDate);
    System.out.println("---------------------------------------------");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testGetRequestIdentify() throws ParseException {
    System.out.println("Test for the verb \"Identify\" with succesfull response");
    String r = this.request.getRequest("Identify", "", "", "", "", "", "");
    System.out.println(r);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testGetRequestIdentifyVerbError() throws ParseException {
    System.out.println("Test for the verb \"Identify\" with error response (MetadaFormat)");
    String t = this.request.getRequest("Identify", "", "a", "", "", "", "");
    System.out.println(t);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testGetRequestIdentifyArgumentError() throws ParseException {
    System.out.println("Test for the verb \"Identify\" with error response (Until and Identifier)");
    String z = this.request.getRequest("Identify", "a", "", "", "", "a", "");
    System.out.println(z);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testGetDateFormat() throws ParseException {
    System.out.println("Test get time format");
    System.out.println(OAIPMHUtilities.getFormatOfDate("2019-12-02T15:36:13Z"));
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testGetRequestGetRecordTextGrid() throws ParseException {
    record.setContributor(TGConstants.CONTRIBUTOR_LIST);
    record.setCoverage(TGConstants.COVERAGE_LIST);
    record.setCreator(TGConstants.CREATOR_LIST);
    record.setDates(TGConstants.DATE_LIST);
    record.setDescriptions(TGConstants.DESCRIPTION_LIST);
    record.setFormats(TGConstants.FORMAT_LIST);
    record.setIdentifiers(TGConstants.IDENTIFIER_LIST);
    record.setLanguages(TGConstants.LANGUAGE_LIST);
    record.setPublishers(TGConstants.PUBLISHER_LIST);
    record.setRelations(TGConstants.RELATIONS_LIST);
    // record.setRelationsForWork(TGConstants.RELATIONS_FOR_WORK_LIST);
    record.setRights(TGConstants.RIGHTS_LIST);
    record.setSources(TGConstants.SOURCE_LIST);
    record.setSubjects(TGConstants.SUBJECT_LIST);
    record.setTitles(TGConstants.TITLE_LIST);
    record.setTypes(TGConstants.TYPE_LIST);
    record.setFields(TGConstants.TGFields);
    record.setFormatField(TGConstants.FORMAT);
    record.setFormatToFilter(TextGridMimetypes.EDITION);
    record.setDateOfObjectCreation(TGConstants.CREATED);
    record.setRelationToFurtherMetadataObject(TGConstants.EDITION_ISEDITIONOF);
    record.setRepositoryObjectURIPrefix(TGConstants.ITEM_IDENTIFIER_PREFIX);
    record.setIdentifierField("textgridUri");

    System.out.println("Test for the verb \"GetRecord\" with succesfull response");
    String p = this.request.getRequest("GetRecord", "textgrid:11hp0.0", "oai_dc", "", "", "", "");
    System.out.println(p);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testGetRequestGetRecordIDIOM() throws ParseException {
    System.out.println("Test for the verb \"GetRecord\" with succesfull response");
    String p = this.request.getRequest("GetRecord", "textgrid:2rcj2",
        OAIPMHConstants.METADATA_IDIOM_PREFIX, "", "", "", "");
    System.out.println(p);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   * @throws JAXBException
   * @throws DatatypeConfigurationException
   * @throws IOException
   */
  @Test
  @Ignore
  public void testGetRequestGetRecordOpenAire()
      throws ParseException, JAXBException, DatatypeConfigurationException, IOException {
    System.out.println("Test for OpenAire get record");

    System.out.println("-----------------------------------\n");
    // OAIPMHUtilities.marshal(oar.getResource());
    // System.out.println(OAIPMHImpl.getStringFromJAXBOAIElement("GetRecord",
    // oar.getRecord("24gv8.0")));

    // OAIPMHUtilities.marshal(oar.getRecord("24gv8.0"));

    String p = this.request.getRequest("GetRecord", "mk7j.0",
        OAIPMHConstants.METADATA_OPENAIRE_PREFIX, "", "", "", "");

    System.out.println(p);
  }

  /**
   * @throws IOException
   */
  @Test
  @Ignore
  public void testLIstRecordsOpenAire() throws IOException {

    recordListDATACITE.setOaiEsClient(oaiEsClient);
    recordListDATACITE.setRangeField(TGConstants.RANGE_FIELD);
    recordListDATACITE.setFormatField(TGConstants.FORMAT);
    recordListDATACITE.setFormatToFilter(TextGridMimetypes.EDITION);
    recordListDATACITE.setSearchResponseSize(100);

    String p = this.request.getRequest("ListRecords", "", OAIPMHConstants.METADATA_OPENAIRE_PREFIX,
        "", "", "", "");

    System.out.println(p);

  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testGetRequestGetRecordErrorIdentifierMissing() throws ParseException {
    System.out.println("Test for the verb \"GetRecord\" with errpr response (Identifier Missing)");
    String p = this.request.getRequest("GetRecord", "", "", "", "", "", "");
    System.out.println(p);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListIdentifierSetsDC() throws ParseException {
    OaiPmhTestTG.identifierList.setFieldForRange(TGConstants.RANGE_FIELD);
    OaiPmhTestTG.identifierList.setIdentifierListFields(TGConstants.IDENTIFIER_LIST_FIELDS);
    OaiPmhTestTG.identifierList.setDateOfObjectCreation(TGConstants.CREATED);
    OaiPmhTestTG.identifierList.setRepositoryObjectURIPrefix(TGConstants.ITEM_IDENTIFIER_PREFIX);
    OaiPmhTestTG.identifierList.setIdentifierField("textgridUri");
    OaiPmhTestTG.identifierList.setSearchResponseSize("100");
    System.out.println("Test for the verb \"ListIdentifiers\" with succesfull response");
    String p =
        this.request.getRequest("ListIdentifiers", "", "oai_dc", "", "2000-02-05", "", "junk");
    System.out.println(p);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListIdentifierWithResumptionTokenDC() throws ParseException {
    OaiPmhTestTG.identifierList.setFieldForRange(TGConstants.RANGE_FIELD);
    OaiPmhTestTG.identifierList.setIdentifierListFields(TGConstants.IDENTIFIER_LIST_FIELDS);
    OaiPmhTestTG.identifierList.setDateOfObjectCreation(TGConstants.CREATED);
    OaiPmhTestTG.identifierList.setRepositoryObjectURIPrefix(TGConstants.ITEM_IDENTIFIER_PREFIX);
    OaiPmhTestTG.identifierList.setIdentifierField("textgridUri");
    OaiPmhTestTG.identifierList.setSearchResponseSize("100");
    System.out.println("Test for the verb \"ListIdentifiers\" with succesfull response");
    String p = this.request.getRequest("ListIdentifiers", "", "oai_dc", "", "", "", "");
    System.out.println(p);
    String resToken = "";
    for (Map.Entry<String, Integer> entry : IdentifierListDelivererDC.cursorCollector.entrySet()) {
      resToken = entry.getKey();
    }
    System.out.println("HASH MAP BEFORE: ");
    System.out.println(IdentifierListDelivererDC.cursorCollector);
    System.out.println(resToken);
    String p2 = this.request.getRequest("ListIdentifiers", "", "", "", "", "", resToken);
    System.out.println(p2);
    System.out.println("HASH MAP AFTER: ");
    System.out.println(IdentifierListDelivererDC.cursorCollector);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListIdentifierIDIOMInvalidResumptionToken() throws ParseException {
    System.out
        .println("Test for the verb \"ListIdentifiers\" in IDIOM with an invalid resumption token");
    String p = this.request.getRequest("ListIdentifiers", "", "", "", "", "",
        "45d8b524-b69e-44e8-81a9-a3634005a430");
    System.out.println(p);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testMissingVerbArgument() throws ParseException {
    System.out
        .println("Test for request with missing verb argument");
    String p = this.request.getRequest("", "", "", "", "", "", "");
    System.out.println(p);
    System.out.println("-----------------------------------\n");
  }


  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListIdentifierIDIOMWithResumptionToken() throws ParseException {
    System.out.println("Test for the verb \"ListIdentifiers\" in IDIOM with successfull response");
    String p = this.request.getRequest("ListIdentifiers", "", OAIPMHConstants.METADATA_IDIOM_PREFIX,
        "", "", "", "");
    System.out.println(p);
    String resToken = "";
    for (Map.Entry<String, Integer> entry : IdentifierListDelivererIDIOM.cursorCollector
        .entrySet()) {
      resToken = entry.getKey();
    }
    String p2 = this.request.getRequest("ListIdentifiers", "", "", "", "", "", resToken);
    System.out.println(p2);
    System.out.println("HASH MAP AFTER: ");
    System.out.println(IdentifierListDelivererIDIOM.cursorCollector);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListRecordSets() throws ParseException {
    OaiPmhTestTG.recordList.setContributors(TGConstants.CONTRIBUTOR_LIST);
    OaiPmhTestTG.recordList.setCoverages(TGConstants.COVERAGE_LIST);
    OaiPmhTestTG.recordList.setCreators(TGConstants.CREATOR_LIST);
    OaiPmhTestTG.recordList.setDates(TGConstants.DATE_LIST);
    OaiPmhTestTG.recordList.setDescriptions(TGConstants.DESCRIPTION_LIST);
    OaiPmhTestTG.recordList.setFormats(TGConstants.FORMAT_LIST);
    OaiPmhTestTG.recordList.setIdentifiers(TGConstants.IDENTIFIER_LIST);
    OaiPmhTestTG.recordList.setLanguages(TGConstants.LANGUAGE_LIST);
    OaiPmhTestTG.recordList.setPublishers(TGConstants.PUBLISHER_LIST);
    OaiPmhTestTG.recordList.setRelations(TGConstants.RELATIONS_LIST);
    OaiPmhTestTG.recordList.setRelationsForWork(TGConstants.RELATIONS_FOR_WORK_LIST);
    OaiPmhTestTG.recordList.setRights(TGConstants.RIGHTS_LIST);
    OaiPmhTestTG.recordList.setSources(TGConstants.SOURCE_LIST);
    OaiPmhTestTG.recordList.setSubjects(TGConstants.SUBJECT_LIST);
    OaiPmhTestTG.recordList.setTitles(TGConstants.TITLE_LIST);
    OaiPmhTestTG.recordList.setTypes(TGConstants.TYPE_LIST);
    OaiPmhTestTG.recordList.setFormatField(TGConstants.FORMAT);
    OaiPmhTestTG.recordList.setFormatToFilter(TextGridMimetypes.EDITION);
    OaiPmhTestTG.recordList.setDateOfObjectCreation(TGConstants.CREATED);
    OaiPmhTestTG.recordList.setRelationToFurtherMetadataObject(TGConstants.EDITION_ISEDITIONOF);
    OaiPmhTestTG.recordList.setRepositoryObjectURIPrefix(TGConstants.ITEM_IDENTIFIER_PREFIX);
    OaiPmhTestTG.recordList.setRangeField(TGConstants.CREATED);
    OaiPmhTestTG.recordList.setModifiedField(TGConstants.MODIFIED_FIELD);
    OaiPmhTestTG.recordList.setIdentifierField("textgridUri");

    System.out.println("Test for the verb \"ListRecords\" with sets with succesfull response");
    // String p = this.request.getRequest("ListRecords", "", "oai_dc",
    // "project:TGPR-f89ad029-4eb2-ae5c-6028-5db876513128", "", "", "");
    String p = this.request.getRequest("ListRecords", "", "oai_dc", "", "", "", "");

    System.out.println(p);
    /*
     * String resToken = ""; for (Map.Entry<String, Integer> entry :
     * RecordListDelivererDC.cursorCollector.entrySet()) { resToken = entry.getKey(); String p2 =
     * this.request.getRequest("ListRecords", "", "", "", "", "", resToken); System.out.println(p2);
     * System.out.println("HASH MAP AFTER: ");
     * System.out.println(RecordListDelivererDC.cursorCollector);
     * System.out.println("-----------------------------------\n"); }
     */
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListRecordWithResumptionTokenForIDIOM() throws ParseException {
    System.out.println("Test for the verb \"ListRecords\" for IDIOM with succesfull response");
    String p = this.request.getRequest("ListRecords", "", OAIPMHConstants.METADATA_IDIOM_PREFIX, "",
        "", "", "");
    System.out.println(p);
    String resToken = "";
    for (Map.Entry<String, Integer> entry : RecordListDelivererIDIOM.cursorCollector.entrySet()) {
      resToken = entry.getKey();
      String p2 = this.request.getRequest("ListRecords", "", "", "", "", "", resToken);
      System.out.println(p2);
      System.out.println("HASH MAP AFTER: ");
      System.out.println(RecordListDelivererIDIOM.cursorCollector);
      System.out.println("-----------------------------------\n");
    }

  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListRecords() throws ParseException {
    String r =
        this.request.getRequest("ListRecords", "", "oai_dc", "", "2011-02-06T20:48:39", "", "");
    System.out.println(r);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListRecords2() throws ParseException {
    System.out.println("HU");
    OaiPmhTestTG.recordList.setContributors(TGConstants.CONTRIBUTOR_LIST);
    OaiPmhTestTG.recordList.setCoverages(TGConstants.COVERAGE_LIST);
    OaiPmhTestTG.recordList.setCreators(TGConstants.CREATOR_LIST);
    OaiPmhTestTG.recordList.setDates(TGConstants.DATE_LIST);
    OaiPmhTestTG.recordList.setDescriptions(TGConstants.DESCRIPTION_LIST);
    OaiPmhTestTG.recordList.setFormats(TGConstants.FORMAT_LIST);
    OaiPmhTestTG.recordList.setIdentifiers(TGConstants.IDENTIFIER_LIST);
    OaiPmhTestTG.recordList.setLanguages(TGConstants.LANGUAGE_LIST);
    OaiPmhTestTG.recordList.setPublishers(TGConstants.PUBLISHER_LIST);
    OaiPmhTestTG.recordList.setRelations(TGConstants.RELATIONS_LIST);
    OaiPmhTestTG.recordList.setRelationsForWork(TGConstants.RELATIONS_FOR_WORK_LIST);
    OaiPmhTestTG.recordList.setRights(TGConstants.RIGHTS_LIST);
    OaiPmhTestTG.recordList.setSources(TGConstants.SOURCE_LIST);
    OaiPmhTestTG.recordList.setSubjects(TGConstants.SUBJECT_LIST);
    OaiPmhTestTG.recordList.setTitles(TGConstants.TITLE_LIST);
    OaiPmhTestTG.recordList.setTypes(TGConstants.TYPE_LIST);
    OaiPmhTestTG.recordList.setFormatField(TGConstants.FORMAT);
    OaiPmhTestTG.recordList.setFormatToFilter(TextGridMimetypes.EDITION);
    OaiPmhTestTG.recordList.setDateOfObjectCreation(TGConstants.CREATED);
    OaiPmhTestTG.recordList.setRelationToFurtherMetadataObject(TGConstants.EDITION_ISEDITIONOF);
    OaiPmhTestTG.recordList.setRepositoryObjectURIPrefix(TGConstants.ITEM_IDENTIFIER_PREFIX);
    OaiPmhTestTG.recordList.setRangeField(TGConstants.CREATED);
    OaiPmhTestTG.recordList.setModifiedField(TGConstants.CREATED);
    OaiPmhTestTG.recordList.setIdentifierField(TGConstants.URI);
    OaiPmhTestTG.recordList.setSearchResponseSize(100);
    String r = this.request.getRequest("ListRecords", "", "oai_dc", "", "", "", "");
    System.out.println(r);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListMetadataFormatsForIdentifier() throws ParseException {
    String r =
        this.request.getRequest("ListMetadataFormats", "textgrid:125x1.0", "", "", "", "", "");
    System.out.println(r);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListMetadataFormatsWithoutID() throws ParseException {
    String r = this.request.getRequest("ListMetadataFormats", "", "", "", "", "", "");
    System.out.println(r);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListSetsTG() throws ParseException {
    OaiPmhTestTG.setListTextGrid.setFormatField("format");
    OaiPmhTestTG.setListTextGrid.setFormatToFilter(TextGridMimetypes.EDITION);
    OaiPmhTestTG.setListTextGrid.setIdentifierField("textgridUri");
    OaiPmhTestTG.setListTextGrid.setRepositoryObjectURIPrefix("textgrid:");
    String r = this.request.getRequest("ListSets", "", "", "", "", "", "");
    System.out.println(r);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListSetsDH() throws ParseException {
    OaiPmhTestTG.setListTextGrid.setFormatField("format");
    OaiPmhTestTG.setListTextGrid.setFormatToFilter(TextGridMimetypes.DARIAH_COLLECTION);
    OaiPmhTestTG.setListTextGrid.setIdentifierField("textgridUri");
    OaiPmhTestTG.setListTextGrid.setRepositoryObjectURIPrefix("textgrid:");
    String r = this.request.getRequest("ListSets", "", "", "", "", "", "");
    System.out.println(r);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testEmptyRequest() throws ParseException {
    OaiPmhTestTG.identifierList.setFieldForRange(TGConstants.RANGE_FIELD);
    OaiPmhTestTG.identifierList.setIdentifierListFields(TGConstants.IDENTIFIER_LIST_FIELDS);
    OaiPmhTestTG.identifierList.setDateOfObjectCreation(TGConstants.CREATED);
    OaiPmhTestTG.identifierList.setIdentifierField("textgridUri");
    OaiPmhTestTG.identifierList.setRepositoryObjectURIPrefix("textgrid:");
    OaiPmhTestTG.identifierList.setSearchResponseSize("100");
    String r = this.request.getRequest("", "", "", "", "", "", "");
    System.out.println(r);
    System.out.println("-----------------------------------\n");
  }

}
